import { __decorate, __metadata, __param } from 'tslib';
import { Injectable, Inject, PLATFORM_ID, NgModule } from '@angular/core';
import { isPlatformBrowser, isPlatformServer, CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { makeStateKey, TransferState, BrowserTransferStateModule } from '@angular/platform-browser';
import { firebase } from '@firebase/app';
import 'firebase/app';
import { Subject, from, BehaviorSubject, of } from 'rxjs';
import 'firebase/auth';
import { map, tap } from 'rxjs/operators';
import 'firebase/database';
import 'firebase/firestore';
import 'firebase/storage';
import 'firebase/messaging';

class FirebaseAppConfig {
}

let AngularFireLiteApp = class AngularFireLiteApp {
    constructor(appConfig) {
        this.appConfig = appConfig;
    }
    instance() {
        let fbApp;
        if (!firebase.apps.length) {
            fbApp = firebase.initializeApp(this.appConfig);
        }
        else {
            fbApp = firebase.app();
        }
        return fbApp;
    }
    config() {
        return this.appConfig;
    }
};
AngularFireLiteApp = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [FirebaseAppConfig])
], AngularFireLiteApp);

let AngularFireLiteAuth = class AngularFireLiteAuth {
    constructor(app, http, platformId) {
        this.app = app;
        this.http = http;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.server = isPlatformServer(this.platformId);
        this.auth = app.instance().auth();
        this.config = app.config();
    }
    // ------------- Authentication Information Getters -----------------//
    uid() {
        const UID = new Subject();
        this.isAuthenticated().subscribe((isAuth) => {
            if (isAuth) {
                this.auth.onAuthStateChanged((user) => {
                    UID.next(user.uid);
                });
            }
            else {
                UID.next(null);
            }
        });
        return UID;
    }
    isAuthenticated() {
        const IS_AUTHENTICATED = new Subject();
        this.auth.onAuthStateChanged((user) => {
            IS_AUTHENTICATED.next(!!user);
        });
        return IS_AUTHENTICATED;
    }
    isAnonymous() {
        const IS_ANONYMOUS = new Subject();
        this.auth.onAuthStateChanged((user) => {
            IS_ANONYMOUS.next(user.isAnonymous);
        });
        return IS_ANONYMOUS;
    }
    currentUser() {
        const CURRENT_USER = new Subject();
        CURRENT_USER.next(this.auth.currentUser);
        return CURRENT_USER;
    }
    userData() {
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/getAccountInfo?key=${this.config.apiKey}`, {
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            const USER_DATA = new Subject();
            this.auth.onAuthStateChanged((user) => {
                if (user) {
                    USER_DATA.next({
                        displayName: user.displayName,
                        email: user.email,
                        emailVerified: user.emailVerified,
                        photoURL: user.photoURL,
                        isAnonymous: user.isAnonymous,
                        uid: user.uid,
                        providerData: user.providerData,
                        phoneNumber: user.phoneNumber
                    });
                }
            });
            return USER_DATA;
        }
    }
    providers(email) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/createAuthUri?key=${this.config.apiKey}`, {
                'identifier': email,
            });
        }
        if (this.browser) {
            const PROVIDERS = new Subject();
            this.auth.fetchSignInMethodsForEmail(email).then(((providers) => {
                PROVIDERS.next(providers);
            }));
            return PROVIDERS;
        }
    }
    // ------------- Authentication Actions -----------------//
    signin(email, password) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/verifyPassword?key=${this.config.apiKey}`, {
                'email': email,
                'password': password,
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.signInWithEmailAndPassword(email, password));
        }
    }
    signinAnonymously() {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/signupNewUser?key=${this.config.apiKey}`, {
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.signInAnonymously());
        }
    }
    signup(email, password) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/signupNewUser?key=${this.config.apiKey}`, {
                'email': email,
                'password': password,
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.createUserWithEmailAndPassword(email, password));
        }
    }
    signout() {
        return from(this.auth.signOut());
    }
    updateProfile(data) {
        if (this.server) {
            let deleteAttribute;
            if (data.displayName === null && data.photoURL === null) {
                deleteAttribute = '\'PHOTO_URL\' , \'DISPLAY_NAME\'';
            }
            else if (data.displayName === null) {
                deleteAttribute = 'DISPLAY_NAME';
            }
            else if (data.photoURL === null) {
                deleteAttribute = 'PHOTO_URL';
            }
            return this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=${this.config.apiKey}`, {
                    'idToken': idToken,
                    'displayName': data.displayName,
                    'photoUrl': data.photoURL,
                    'deleteAttribute': deleteAttribute,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updateProfile(data));
        }
    }
    updateEmail(newEmail) {
        if (this.server) {
            return this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=${this.config.apiKey}`, {
                    'idToken': idToken,
                    'email': newEmail,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updateEmail(newEmail));
        }
    }
    updatePassword(newPassword) {
        if (this.server) {
            return this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=${this.config.apiKey}`, {
                    'idToken': idToken,
                    'password': newPassword,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updatePassword(newPassword));
        }
    }
    verifyPasswordResetCode(code) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/resetPassword?key=${this.config.apiKey}`, {
                'oobCode': code
            });
        }
        if (this.browser) {
            return from(this.auth.verifyPasswordResetCode(code));
        }
    }
    confirmPasswordReset(code, newPassword) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/resetPassword?key=${this.config.apiKey}`, {
                'oobCode': code,
                'newPassword': newPassword
            });
        }
        if (this.browser) {
            return from(this.auth.confirmPasswordReset(code, newPassword));
        }
    }
    relogin(credentials) {
        return from(this.auth.currentUser.reauthenticateWithCredential(credentials));
    }
    deletePermanently() {
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/deleteAccount?key=${this.config.apiKey}`, {
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            return from(this.auth.currentUser.delete());
        }
    }
    // ------------- Authentication Emails Senders -----------------//
    sendEmailVerification() {
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then((idToken) => {
                return this.http
                    .post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/getOobConfirmationCode?key=${this.config.apiKey}`, {
                    'requestType': 'VERIFY_EMAIL',
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            return from(this.auth.currentUser.sendEmailVerification());
        }
    }
    // TODO: Confirm email verification
    sendPasswordResetEmail(email) {
        if (this.server) {
            return this.http.post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/getOobConfirmationCode?key=${this.config.apiKey}`, {
                'requestType': 'PASSWORD_RESET',
                'email': email
            });
        }
        if (this.browser) {
            return from(this.auth.sendPasswordResetEmail(email));
        }
    }
};
AngularFireLiteAuth = __decorate([
    Injectable(),
    __param(2, Inject(PLATFORM_ID)),
    __metadata("design:paramtypes", [AngularFireLiteApp,
        HttpClient,
        Object])
], AngularFireLiteAuth);

let AngularFireLiteDatabase = class AngularFireLiteDatabase {
    constructor(app, http, state, platformId) {
        this.app = app;
        this.http = http;
        this.state = state;
        this.platformId = platformId;
        this.server = isPlatformServer(this.platformId);
        this.browser = isPlatformBrowser(this.platformId);
        this.ref = (ref) => this.database.ref(ref);
        this.database = app.instance().database();
        this.config = app.config();
    }
    // ------------- Read -----------------//
    read(ref) {
        const dataStateKey = makeStateKey(ref + 'rtd:read');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'value', dataStateKey);
        }
    }
    childAdded(ref) {
        const dataStateKey = makeStateKey(ref + 'rtd:child_added');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_added', dataStateKey);
        }
    }
    childChanged(ref) {
        const dataStateKey = makeStateKey(ref + 'rtd:child_changed');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_changed', dataStateKey);
        }
    }
    childRemoved(ref) {
        const dataStateKey = makeStateKey(ref + 'rtd:child_removed');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_removed', dataStateKey);
        }
    }
    childMoved(ref) {
        const dataStateKey = makeStateKey(ref + 'rtd:child_moved');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_moved', dataStateKey);
        }
    }
    write(ref, data) {
        if (this.server) {
            return this.http.put(`https://${this.config.projectId}.firebaseio.com/${ref}.json?print=silent`, data);
        }
        if (this.browser) {
            return from(this.ref(ref).set(data));
        }
    }
    push(ref, data) {
        if (this.server) {
            return this.http.post(`https://${this.config.projectId}.firebaseio.com/${ref}.json?print=silent`, data);
        }
        if (this.browser) {
            return from(this.ref(ref).push(data));
        }
    }
    // ------------- Write -----------------//
    update(ref, data) {
        if (this.server) {
            return this.http.patch(`https://${this.config.projectId}.firebaseio.com/${ref}.json?print=silent`, data);
        }
        if (this.browser) {
            return from(this.ref(ref).update(data));
        }
    }
    remove(ref) {
        if (this.server) {
            return this.http.delete(`https://${this.config.projectId}.firebaseio.com/${ref}.json`);
        }
        if (this.browser) {
            return from(this.ref(ref).remove());
        }
    }
    query(ref) {
        const PID = this.platformId;
        const state = this.state;
        const db = this.database;
        const http = this.http;
        const config = this.config;
        const SQH = function (REF, FSQ, DSK) {
            return http.get(`https://${config.projectId}.firebaseio.com/${REF}.json?${FSQ}`)
                .pipe(map((payload) => {
                if (!!payload && typeof payload === 'object') {
                    const result = [];
                    Object.keys(payload).forEach((key) => {
                        result.push(payload[key]);
                    });
                    state.set(DSK, result);
                    return result;
                }
                else {
                    state.set(DSK, payload);
                    return payload;
                }
            }));
        };
        const BQH = function (BS) {
            return (snapshot) => {
                if (!!snapshot.val() && typeof snapshot.val() === 'object') {
                    const result = Object.keys(snapshot.val()).map(function (key) {
                        return [snapshot.val()[key]];
                    });
                    BS.next(result);
                }
                else {
                    BS.next(snapshot.val());
                }
            };
        };
        let SQ = '';
        let BQ = this.ref(ref);
        return {
            startAt(value) {
                SQ += `&startAt=${value}`;
                BQ = BQ.startAt(value);
                return this;
            },
            endAt(value) {
                SQ += `&endAt=${value}`;
                BQ = BQ.endAt(value);
                return this;
            },
            equalTo(value) {
                SQ += `&equalTo=${value}`;
                BQ = BQ.equalTo(value);
                return this;
            },
            isEqual(query) {
                if (isPlatformBrowser(PID)) {
                    return db.ref(ref).isEqual(query);
                }
            },
            limitToFirst(limit) {
                SQ += `&limitToFirst=${limit}`;
                BQ = BQ.limitToFirst(limit);
                return this;
            },
            limitToLast(limit) {
                SQ += `&limitToLast=${limit}`;
                BQ = BQ.limitToLast(limit);
                return this;
            },
            orderByChild(path) {
                SQ += `&orderBy="${path}"`;
                BQ = BQ.orderByChild(path);
                return this;
            },
            orderByKey() {
                SQ += `&orderBy="$key"`;
                BQ = BQ.orderByKey();
                return this;
            },
            orderByPriority() {
                SQ += `&orderBy="$priority"`;
                BQ = BQ.orderByPriority();
                return this;
            },
            orderByValue() {
                SQ += `&orderBy="$value"`;
                BQ = BQ.orderByValue();
                return this;
            },
            on(event) {
                const dataStateKey = makeStateKey(ref + 'rtd');
                if (isPlatformServer(PID)) {
                    return SQH(ref, SQ, dataStateKey);
                }
                if (isPlatformBrowser(PID)) {
                    const SSRedValue = state.get(dataStateKey, []);
                    const VALUE = new BehaviorSubject(SSRedValue);
                    BQ.on(event, BQH(VALUE));
                    return VALUE;
                }
            },
            once(event) {
                const dataStateKey = makeStateKey(ref + 'rtd');
                if (isPlatformServer(PID)) {
                    return SQH(ref, SQ, dataStateKey);
                }
                if (isPlatformBrowser(PID)) {
                    const SSRedValue = state.get(dataStateKey, []);
                    const VALUE = new BehaviorSubject(SSRedValue);
                    BQ.once(event, BQH(VALUE));
                    return VALUE;
                }
            }
        };
    }
    SRH(ref, DSK) {
        if (this.server) {
            return this.http.get(`https://${this.config.projectId}.firebaseio.com/${ref}.json`)
                .pipe(map((payload) => {
                if (!!payload && typeof payload === 'object') {
                    const result = Object.keys(payload).map((key) => {
                        return [payload[key]];
                    });
                    this.state.set(DSK, result);
                    return result;
                }
                else {
                    this.state.set(DSK, payload);
                    return payload;
                }
            }));
        }
    }
    BRH(ref, event, DSK) {
        if (this.browser) {
            const SSRedValue = this.state.get(DSK, []);
            const DATA = new BehaviorSubject(SSRedValue);
            this.ref(ref).on(event, (snapshot) => {
                if (!!snapshot.val() && typeof snapshot.val() === 'object') {
                    const result = Object.keys(snapshot.val()).map(function (key) {
                        return [snapshot.val()[key]];
                    });
                    DATA.next(result);
                }
                else {
                    DATA.next(snapshot.val());
                }
            });
            return DATA;
        }
    }
};
AngularFireLiteDatabase = __decorate([
    Injectable(),
    __param(3, Inject(PLATFORM_ID)),
    __metadata("design:paramtypes", [AngularFireLiteApp,
        HttpClient,
        TransferState,
        Object])
], AngularFireLiteDatabase);

let AngularFireLiteFirestore = class AngularFireLiteFirestore {
    constructor(app, http, state, platformId) {
        this.app = app;
        this.http = http;
        this.state = state;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.firestore = app.instance().firestore();
        this.config = app.config();
    }
    // ------------- Read -----------------//
    read(ref) {
        const dataStateKey = makeStateKey(ref + 'firestore:read');
        const refArray = ref.split('/');
        if (refArray[0] === '/') {
            refArray.shift();
        }
        if (refArray[refArray.length - 1] === '/') {
            refArray.pop();
        }
        const slashes = refArray.length - 1;
        if (isPlatformServer(this.platformId)) {
            return this.http
                .get(`https://firestore.googleapis.com/v1beta1/projects/${this.config.projectId}/databases/(default)/documents/${ref}`)
                .pipe(map((res) => {
                const docData = {};
                if (slashes % 2 !== 0) {
                    Object.keys(res.fields)
                        .forEach((key) => {
                        for (const keyValue in res.fields[key]) {
                            if (keyValue) {
                                docData[key] = res.fields[key][keyValue];
                            }
                        }
                    });
                    return docData;
                }
                else {
                    const colData = [];
                    res.documents.forEach((doc) => {
                        const singleDocData = {};
                        Object.keys(doc.fields)
                            .forEach((key) => {
                            for (const keyValue in doc.fields[key]) {
                                if (keyValue) {
                                    singleDocData[key] = doc.fields[key][keyValue];
                                }
                            }
                        });
                        colData.push(singleDocData);
                    });
                    return colData;
                }
            }), tap((pl) => {
                this.state.set(dataStateKey, pl);
            }));
        }
        if (this.browser) {
            const data = [];
            const SSRedValue = this.state.get(dataStateKey, []);
            const DATA = new BehaviorSubject(SSRedValue);
            if (slashes % 2 === 0) {
                this.firestore.collection(ref).onSnapshot((snapshot) => {
                    snapshot.docs.forEach((doc) => {
                        data.push(doc.data());
                    });
                    DATA.next(data);
                });
            }
            else {
                this.firestore.doc(ref).onSnapshot((snapshot) => {
                    DATA.next(snapshot.data());
                });
            }
            return DATA;
        }
    }
    // ------------- Write -----------------//
    write(ref, data, merge) {
        if (this.browser) {
            return from(this.firestore.doc(ref).set(data, { merge: merge }));
        }
    }
    push(ref, data) {
        if (this.browser) {
            return from(this.firestore.collection(ref).add(data));
        }
    }
    update(ref, data) {
        if (this.browser) {
            return from(this.firestore.doc(ref).update(data));
        }
    }
    // ------------- Delete -----------------//
    remove(DocumentRef) {
        if (isPlatformServer(this.platformId)) {
            return this.http
                .delete(`https://firestore.googleapis.com/v1beta1/projects/${this.config.projectId}/databases/(default)/documents/${DocumentRef}`);
        }
        if (this.browser) {
            return from(this.firestore.doc(DocumentRef).delete());
        }
    }
    removeField(ref, ...fields) {
        if (this.browser) {
            return from(this.firestore.doc(ref).update(fields));
        }
    }
    removeCollection(collectionRef) {
        if (this.browser) {
            return from(this.firestore.collection(collectionRef).get().then((snapshot) => {
                snapshot.docs.forEach((doc) => {
                    this.firestore.batch().delete(doc.ref);
                });
            }));
        }
    }
    // ------------- Query -----------------//
    query(ref) {
        const PID = this.platformId;
        const HTTP = this.http;
        const CONFIG = this.config;
        const state = this.state;
        const fs = this.firestore;
        const SSQ = {
            'from': [{ 'collectionId': ref }]
        };
        const SQ = {
            'structuredQuery': SSQ
        };
        function SQHFS(DSK) {
            const data = [];
            return HTTP
                .post(`https://firestore.googleapis.com/v1beta1/projects/${CONFIG.projectId}/databases/(default)/documents:runQuery`, JSON.stringify(SQ))
                .pipe(map((res) => {
                for (const doc of res) {
                    const documentData = {};
                    Object.keys(doc.document.fields).forEach((fieldName) => {
                        const fieldType = Object.keys(doc.document.fields[fieldName]);
                        documentData[fieldName] = doc.document.fields[fieldName][fieldType[0]];
                    });
                    data.push(documentData);
                }
                return data;
            }), tap((pl) => {
                state.set(DSK, pl);
            }));
        }
        function FSD(value) {
            switch (typeof value) {
                case 'boolean':
                    return { 'booleanValue': value };
                case 'string':
                    return { 'stringValue': value };
                case 'number':
                    return { 'doubleValue': value };
                case 'object':
                    return { 'arrayValue': value };
            }
        }
        const SQOB = [];
        let BQ = fs.collection(ref);
        return {
            where(document, comparison, value) {
                let SOP = '';
                switch (comparison) {
                    case '<':
                        SOP = 'LESS_THAN';
                        break;
                    case '<=':
                        SOP = 'LESS_THAN_OR_EQUAL';
                        break;
                    case '>':
                        SOP = 'GREATER_THAN';
                        break;
                    case '>=':
                        SOP = 'GREATER_THAN_OR_EQUAL';
                        break;
                    case '==':
                        SOP = 'EQUAL';
                        break;
                }
                SSQ['where'] = { 'fieldFilter': { 'field': { 'fieldPath': document }, 'op': SOP, 'value': FSD(value) } };
                BQ = BQ.where(document, comparison, value);
                return this;
            },
            startAt(...startValue) {
                const SV = [];
                startValue.forEach((value) => {
                    SV.push(FSD(value));
                });
                SSQ.startAt = {};
                SSQ.startAt.before = true;
                SSQ.startAt.values = SV;
                BQ = BQ.startAt(...startValue);
                return this;
            },
            startAfter(...startValue) {
                const SV = [];
                startValue.forEach((value) => {
                    SV.push(FSD(value));
                });
                SSQ.startAt = {};
                SSQ.startAt.before = false;
                SSQ.startAt.values = SV;
                BQ = BQ.startAfter(...startValue);
                return this;
            },
            endAt(...endValue) {
                const SV = [];
                endValue.forEach((value) => {
                    SV.push(FSD(value));
                });
                SSQ.endAt = {};
                SSQ.endAt.before = false;
                SSQ.endAt.values = SV;
                BQ = BQ.endAt(...endValue);
                return this;
            },
            endBefore(...endValue) {
                const SV = [];
                endValue.forEach((value) => {
                    SV.push(FSD(value));
                });
                SSQ.endAt = {};
                SSQ.endAt.before = true;
                SSQ.endAt.values = SV;
                BQ = BQ.endBefore(...endValue);
                return this;
            },
            limit(limit) {
                SSQ.limit = limit;
                BQ = BQ.limit(limit);
                return this;
            },
            orderBy(path, order) {
                const orderBy = {
                    field: {
                        fieldPath: ''
                    }
                };
                orderBy.field.fieldPath = path;
                switch (order) {
                    case 'asc':
                        orderBy['direction'] = 'ASCENDING';
                        break;
                    case 'desc':
                        orderBy['direction'] = 'DESCENDING';
                        break;
                }
                SQOB.push(orderBy);
                BQ = BQ.orderBy(path, order);
                SSQ.orderBy = SQOB;
                return this;
            },
            on() {
                const ONDSK = makeStateKey(ref + 'firestore:query');
                if (isPlatformServer(PID)) {
                    return SQHFS(ONDSK);
                }
                if (isPlatformBrowser(PID)) {
                    const SSRedValue = state.get(ONDSK, []);
                    const VALUE = new BehaviorSubject(SSRedValue);
                    BQ.onSnapshot((snapshot) => {
                        const data = [];
                        snapshot.forEach((doc) => {
                            data.push(doc.data());
                        });
                        VALUE.next(data);
                    });
                    return VALUE;
                }
            },
            get() {
                const GETDSK = makeStateKey(ref + 'firestore:query');
                if (isPlatformServer(PID)) {
                    return SQHFS(GETDSK);
                }
                if (isPlatformBrowser(PID)) {
                    const data = [];
                    const SSRedValue = state.get(GETDSK, []);
                    const VALUE = new BehaviorSubject(SSRedValue);
                    BQ.get().then((snapshot) => {
                        snapshot.forEach((doc) => {
                            data.push(doc);
                        });
                        VALUE.next(data);
                    });
                    return VALUE;
                }
            }
        };
    }
    // ------------- Transactions and Batched Writes -----------------//
    // TODO: Add Transactions SSR support
    transaction() {
        if (this.browser) {
            const fs = this.firestore;
            let transactionToRun;
            let readCount = 0;
            const transactions = {
                get(ref) {
                    return fs.doc(ref).get();
                },
                set(ref, data) {
                    return fs.doc(ref).set(data);
                }
            };
            return {
                set(ref, data) {
                    transactionToRun = transactionToRun.then(() => {
                        transactions.set(ref, data);
                    });
                    return this;
                },
                get(ref) {
                    const getSubject = new Subject();
                    if (readCount > 0) {
                        transactionToRun = transactionToRun.then(() => {
                            transactions.get(ref).then((value) => {
                                getSubject.next({ data: value.data(), next: this });
                            });
                        });
                    }
                    else if (readCount === 0) {
                        transactionToRun = transactions.get(ref).then((value) => {
                            getSubject.next({ data: value.data(), next: this });
                        });
                    }
                    readCount++;
                    return getSubject;
                },
                run() {
                    return from(fs.runTransaction(() => {
                        return transactionToRun;
                    }));
                }
            };
        }
        else {
            return {
                set(ref, data) {
                    return this;
                },
                get(ref) {
                    return new Subject();
                },
                run() {
                    return of();
                }
            };
        }
    }
    // TODO: Add batched writes SSR support
    batch() {
        if (this.browser) {
            const fs = this.firestore;
            const b = this.firestore.batch();
            return {
                set(ref, data) {
                    b.set(fs.doc(ref), data);
                    return this;
                },
                update(ref, data) {
                    b.update(fs.doc(ref), data);
                    return this;
                },
                delete(ref) {
                    b.delete(fs.doc(ref));
                    return this;
                },
                commit() {
                    return from(b.commit());
                }
            };
        }
        else {
            return {
                set(ref, data) {
                    return this;
                },
                update(ref, data) {
                    return this;
                },
                delete(ref) {
                    return this;
                },
                commit() {
                    return of();
                }
            };
        }
    }
};
AngularFireLiteFirestore = __decorate([
    Injectable(),
    __param(3, Inject(PLATFORM_ID)),
    __metadata("design:paramtypes", [AngularFireLiteApp,
        HttpClient,
        TransferState,
        Object])
], AngularFireLiteFirestore);

let AngularFireLiteStorage = class AngularFireLiteStorage {
    constructor(app, platformId) {
        this.app = app;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.storage = app.instance().storage();
    }
    // TODO: Add Server Side Rendering Support Using The GCS REST API
    // ------------- Upload -----------------//
    upload(ref, file, metadata) {
        if (this.browser) {
            return from(this.child(ref).put(file, metadata));
        }
    }
    uploadString(ref, string) {
        if (this.browser) {
            return from(this.child(ref).putString(string));
        }
    }
    // ------------- Download -----------------//
    getDownloadURL(ref) {
        if (this.browser) {
            return from(this.child(ref).getDownloadURL());
        }
    }
    // ------------- Delete -----------------//
    remove(ref) {
        if (this.browser) {
            return from(this.child(ref).delete());
        }
    }
    // ------------- Metadata -----------------//
    getMetadata(ref) {
        if (this.browser) {
            const META = new Subject();
            this.child(ref).getMetadata().then((meta) => {
                META.next(meta);
            });
            return META;
        }
    }
    updateMetadata(ref, metadata) {
        if (this.browser) {
            return from(this.child(ref).updateMetadata(metadata));
        }
    }
    deleteMetadata(ref) {
        if (this.browser) {
            return from(this.child(ref).updateMetadata({
                customMetadata: null,
                cacheControl: null,
                contentEncoding: null,
                contentLanguage: null,
                contentType: null
            }));
        }
    }
    child(ref) {
        return this.storage.ref().child(ref);
    }
};
AngularFireLiteStorage = __decorate([
    Injectable(),
    __param(1, Inject(PLATFORM_ID)),
    __metadata("design:paramtypes", [AngularFireLiteApp,
        Object])
], AngularFireLiteStorage);

let AngularFireLiteMessaging = class AngularFireLiteMessaging {
    constructor(app, platformId) {
        this.app = app;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.messaging = app.instance().messaging();
    }
    // ------------- Getters -----------------//
    instance() {
        if (this.browser) {
            return this.messaging;
        }
    }
    token() {
        if (this.browser) {
            return from(this.messaging.getToken());
        }
    }
    tokenRefresh() {
        if (this.browser) {
            const REFRESH_TOKEN = new Subject();
            this.messaging.onMessage(() => {
                this.messaging.getToken().then((token) => {
                    REFRESH_TOKEN.next(token);
                });
            });
            return REFRESH_TOKEN;
        }
    }
    // ------------- Actions -----------------//
    requestPermission() {
        if (this.browser) {
            return from(this.messaging.requestPermission());
        }
    }
    deleteToken(token) {
        if (this.browser) {
            return from((this.messaging.deleteToken(token)));
        }
    }
};
AngularFireLiteMessaging = __decorate([
    Injectable(),
    __param(1, Inject(PLATFORM_ID)),
    __metadata("design:paramtypes", [AngularFireLiteApp,
        Object])
], AngularFireLiteMessaging);

var AngularFireLite_1;
function AngularFireLiteAppFactory(config) {
    return new AngularFireLiteApp(config);
}
// @dynamic
let AngularFireLite = AngularFireLite_1 = class AngularFireLite {
    static forRoot(fireConfig) {
        return {
            ngModule: AngularFireLite_1,
            providers: [
                { provide: FirebaseAppConfig, useValue: fireConfig },
                {
                    provide: AngularFireLiteApp,
                    useFactory: AngularFireLiteAppFactory,
                    deps: [FirebaseAppConfig]
                },
                AngularFireLiteFirestore,
                AngularFireLiteDatabase,
                AngularFireLiteAuth,
                AngularFireLiteStorage,
                AngularFireLiteMessaging
            ]
        };
    }
};
AngularFireLite = AngularFireLite_1 = __decorate([
    NgModule({
        imports: [
            HttpClientModule,
            CommonModule,
            BrowserTransferStateModule
        ]
    })
], AngularFireLite);

export { AngularFireLite, AngularFireLiteApp, AngularFireLiteAppFactory, AngularFireLiteAuth, AngularFireLiteDatabase, AngularFireLiteFirestore, AngularFireLiteMessaging, AngularFireLiteStorage, FirebaseAppConfig as ɵa };
//# sourceMappingURL=angularfire-lite.js.map
