import { __decorate, __metadata, __param, __spread, __values } from 'tslib';
import { Injectable, Inject, PLATFORM_ID, NgModule } from '@angular/core';
import { isPlatformBrowser, isPlatformServer, CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { makeStateKey, TransferState, BrowserTransferStateModule } from '@angular/platform-browser';
import { firebase } from '@firebase/app';
import 'firebase/app';
import { Subject, from, BehaviorSubject, of } from 'rxjs';
import 'firebase/auth';
import { map, tap } from 'rxjs/operators';
import 'firebase/database';
import 'firebase/firestore';
import 'firebase/storage';
import 'firebase/messaging';

var FirebaseAppConfig = /** @class */ (function () {
    function FirebaseAppConfig() {
    }
    return FirebaseAppConfig;
}());

var AngularFireLiteApp = /** @class */ (function () {
    function AngularFireLiteApp(appConfig) {
        this.appConfig = appConfig;
    }
    AngularFireLiteApp.prototype.instance = function () {
        var fbApp;
        if (!firebase.apps.length) {
            fbApp = firebase.initializeApp(this.appConfig);
        }
        else {
            fbApp = firebase.app();
        }
        return fbApp;
    };
    AngularFireLiteApp.prototype.config = function () {
        return this.appConfig;
    };
    AngularFireLiteApp = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [FirebaseAppConfig])
    ], AngularFireLiteApp);
    return AngularFireLiteApp;
}());

var AngularFireLiteAuth = /** @class */ (function () {
    function AngularFireLiteAuth(app, http, platformId) {
        this.app = app;
        this.http = http;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.server = isPlatformServer(this.platformId);
        this.auth = app.instance().auth();
        this.config = app.config();
    }
    // ------------- Authentication Information Getters -----------------//
    AngularFireLiteAuth.prototype.uid = function () {
        var _this = this;
        var UID = new Subject();
        this.isAuthenticated().subscribe(function (isAuth) {
            if (isAuth) {
                _this.auth.onAuthStateChanged(function (user) {
                    UID.next(user.uid);
                });
            }
            else {
                UID.next(null);
            }
        });
        return UID;
    };
    AngularFireLiteAuth.prototype.isAuthenticated = function () {
        var IS_AUTHENTICATED = new Subject();
        this.auth.onAuthStateChanged(function (user) {
            IS_AUTHENTICATED.next(!!user);
        });
        return IS_AUTHENTICATED;
    };
    AngularFireLiteAuth.prototype.isAnonymous = function () {
        var IS_ANONYMOUS = new Subject();
        this.auth.onAuthStateChanged(function (user) {
            IS_ANONYMOUS.next(user.isAnonymous);
        });
        return IS_ANONYMOUS;
    };
    AngularFireLiteAuth.prototype.currentUser = function () {
        var CURRENT_USER = new Subject();
        CURRENT_USER.next(this.auth.currentUser);
        return CURRENT_USER;
    };
    AngularFireLiteAuth.prototype.userData = function () {
        var _this = this;
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/getAccountInfo?key=" + _this.config.apiKey, {
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            var USER_DATA_1 = new Subject();
            this.auth.onAuthStateChanged(function (user) {
                if (user) {
                    USER_DATA_1.next({
                        displayName: user.displayName,
                        email: user.email,
                        emailVerified: user.emailVerified,
                        photoURL: user.photoURL,
                        isAnonymous: user.isAnonymous,
                        uid: user.uid,
                        providerData: user.providerData,
                        phoneNumber: user.phoneNumber
                    });
                }
            });
            return USER_DATA_1;
        }
    };
    AngularFireLiteAuth.prototype.providers = function (email) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/createAuthUri?key=" + this.config.apiKey, {
                'identifier': email,
            });
        }
        if (this.browser) {
            var PROVIDERS_1 = new Subject();
            this.auth.fetchSignInMethodsForEmail(email).then((function (providers) {
                PROVIDERS_1.next(providers);
            }));
            return PROVIDERS_1;
        }
    };
    // ------------- Authentication Actions -----------------//
    AngularFireLiteAuth.prototype.signin = function (email, password) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/verifyPassword?key=" + this.config.apiKey, {
                'email': email,
                'password': password,
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.signInWithEmailAndPassword(email, password));
        }
    };
    AngularFireLiteAuth.prototype.signinAnonymously = function () {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/signupNewUser?key=" + this.config.apiKey, {
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.signInAnonymously());
        }
    };
    AngularFireLiteAuth.prototype.signup = function (email, password) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/signupNewUser?key=" + this.config.apiKey, {
                'email': email,
                'password': password,
                'returnSecureToken': true
            });
        }
        if (this.browser) {
            return from(this.auth.createUserWithEmailAndPassword(email, password));
        }
    };
    AngularFireLiteAuth.prototype.signout = function () {
        return from(this.auth.signOut());
    };
    AngularFireLiteAuth.prototype.updateProfile = function (data) {
        var _this = this;
        if (this.server) {
            var deleteAttribute_1;
            if (data.displayName === null && data.photoURL === null) {
                deleteAttribute_1 = '\'PHOTO_URL\' , \'DISPLAY_NAME\'';
            }
            else if (data.displayName === null) {
                deleteAttribute_1 = 'DISPLAY_NAME';
            }
            else if (data.photoURL === null) {
                deleteAttribute_1 = 'PHOTO_URL';
            }
            return this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=" + _this.config.apiKey, {
                    'idToken': idToken,
                    'displayName': data.displayName,
                    'photoUrl': data.photoURL,
                    'deleteAttribute': deleteAttribute_1,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updateProfile(data));
        }
    };
    AngularFireLiteAuth.prototype.updateEmail = function (newEmail) {
        var _this = this;
        if (this.server) {
            return this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=" + _this.config.apiKey, {
                    'idToken': idToken,
                    'email': newEmail,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updateEmail(newEmail));
        }
    };
    AngularFireLiteAuth.prototype.updatePassword = function (newPassword) {
        var _this = this;
        if (this.server) {
            return this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/setAccountInfo?key=" + _this.config.apiKey, {
                    'idToken': idToken,
                    'password': newPassword,
                    'returnSecureToken': true
                });
            });
        }
        if (this.browser) {
            return from(this.auth.currentUser.updatePassword(newPassword));
        }
    };
    AngularFireLiteAuth.prototype.verifyPasswordResetCode = function (code) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/resetPassword?key=" + this.config.apiKey, {
                'oobCode': code
            });
        }
        if (this.browser) {
            return from(this.auth.verifyPasswordResetCode(code));
        }
    };
    AngularFireLiteAuth.prototype.confirmPasswordReset = function (code, newPassword) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/resetPassword?key=" + this.config.apiKey, {
                'oobCode': code,
                'newPassword': newPassword
            });
        }
        if (this.browser) {
            return from(this.auth.confirmPasswordReset(code, newPassword));
        }
    };
    AngularFireLiteAuth.prototype.relogin = function (credentials) {
        return from(this.auth.currentUser.reauthenticateWithCredential(credentials));
    };
    AngularFireLiteAuth.prototype.deletePermanently = function () {
        var _this = this;
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/deleteAccount?key=" + _this.config.apiKey, {
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            return from(this.auth.currentUser.delete());
        }
    };
    // ------------- Authentication Emails Senders -----------------//
    AngularFireLiteAuth.prototype.sendEmailVerification = function () {
        var _this = this;
        if (this.server) {
            return from(this.auth.currentUser.getIdToken(true).then(function (idToken) {
                return _this.http
                    .post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/getOobConfirmationCode?key=" + _this.config.apiKey, {
                    'requestType': 'VERIFY_EMAIL',
                    'idToken': idToken
                });
            }));
        }
        if (this.browser) {
            return from(this.auth.currentUser.sendEmailVerification());
        }
    };
    // TODO: Confirm email verification
    AngularFireLiteAuth.prototype.sendPasswordResetEmail = function (email) {
        if (this.server) {
            return this.http.post("https://www.googleapis.com/identitytoolkit/v3/relyingparty/getOobConfirmationCode?key=" + this.config.apiKey, {
                'requestType': 'PASSWORD_RESET',
                'email': email
            });
        }
        if (this.browser) {
            return from(this.auth.sendPasswordResetEmail(email));
        }
    };
    AngularFireLiteAuth = __decorate([
        Injectable(),
        __param(2, Inject(PLATFORM_ID)),
        __metadata("design:paramtypes", [AngularFireLiteApp,
            HttpClient,
            Object])
    ], AngularFireLiteAuth);
    return AngularFireLiteAuth;
}());

var AngularFireLiteDatabase = /** @class */ (function () {
    function AngularFireLiteDatabase(app, http, state, platformId) {
        var _this = this;
        this.app = app;
        this.http = http;
        this.state = state;
        this.platformId = platformId;
        this.server = isPlatformServer(this.platformId);
        this.browser = isPlatformBrowser(this.platformId);
        this.ref = function (ref) { return _this.database.ref(ref); };
        this.database = app.instance().database();
        this.config = app.config();
    }
    // ------------- Read -----------------//
    AngularFireLiteDatabase.prototype.read = function (ref) {
        var dataStateKey = makeStateKey(ref + 'rtd:read');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'value', dataStateKey);
        }
    };
    AngularFireLiteDatabase.prototype.childAdded = function (ref) {
        var dataStateKey = makeStateKey(ref + 'rtd:child_added');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_added', dataStateKey);
        }
    };
    AngularFireLiteDatabase.prototype.childChanged = function (ref) {
        var dataStateKey = makeStateKey(ref + 'rtd:child_changed');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_changed', dataStateKey);
        }
    };
    AngularFireLiteDatabase.prototype.childRemoved = function (ref) {
        var dataStateKey = makeStateKey(ref + 'rtd:child_removed');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_removed', dataStateKey);
        }
    };
    AngularFireLiteDatabase.prototype.childMoved = function (ref) {
        var dataStateKey = makeStateKey(ref + 'rtd:child_moved');
        if (this.server) {
            return this.SRH(ref, dataStateKey);
        }
        if (this.browser) {
            return this.BRH(ref, 'child_moved', dataStateKey);
        }
    };
    AngularFireLiteDatabase.prototype.write = function (ref, data) {
        if (this.server) {
            return this.http.put("https://" + this.config.projectId + ".firebaseio.com/" + ref + ".json?print=silent", data);
        }
        if (this.browser) {
            return from(this.ref(ref).set(data));
        }
    };
    AngularFireLiteDatabase.prototype.push = function (ref, data) {
        if (this.server) {
            return this.http.post("https://" + this.config.projectId + ".firebaseio.com/" + ref + ".json?print=silent", data);
        }
        if (this.browser) {
            return from(this.ref(ref).push(data));
        }
    };
    // ------------- Write -----------------//
    AngularFireLiteDatabase.prototype.update = function (ref, data) {
        if (this.server) {
            return this.http.patch("https://" + this.config.projectId + ".firebaseio.com/" + ref + ".json?print=silent", data);
        }
        if (this.browser) {
            return from(this.ref(ref).update(data));
        }
    };
    AngularFireLiteDatabase.prototype.remove = function (ref) {
        if (this.server) {
            return this.http.delete("https://" + this.config.projectId + ".firebaseio.com/" + ref + ".json");
        }
        if (this.browser) {
            return from(this.ref(ref).remove());
        }
    };
    AngularFireLiteDatabase.prototype.query = function (ref) {
        var PID = this.platformId;
        var state = this.state;
        var db = this.database;
        var http = this.http;
        var config = this.config;
        var SQH = function (REF, FSQ, DSK) {
            return http.get("https://" + config.projectId + ".firebaseio.com/" + REF + ".json?" + FSQ)
                .pipe(map(function (payload) {
                if (!!payload && typeof payload === 'object') {
                    var result_1 = [];
                    Object.keys(payload).forEach(function (key) {
                        result_1.push(payload[key]);
                    });
                    state.set(DSK, result_1);
                    return result_1;
                }
                else {
                    state.set(DSK, payload);
                    return payload;
                }
            }));
        };
        var BQH = function (BS) {
            return function (snapshot) {
                if (!!snapshot.val() && typeof snapshot.val() === 'object') {
                    var result = Object.keys(snapshot.val()).map(function (key) {
                        return [snapshot.val()[key]];
                    });
                    BS.next(result);
                }
                else {
                    BS.next(snapshot.val());
                }
            };
        };
        var SQ = '';
        var BQ = this.ref(ref);
        return {
            startAt: function (value) {
                SQ += "&startAt=" + value;
                BQ = BQ.startAt(value);
                return this;
            },
            endAt: function (value) {
                SQ += "&endAt=" + value;
                BQ = BQ.endAt(value);
                return this;
            },
            equalTo: function (value) {
                SQ += "&equalTo=" + value;
                BQ = BQ.equalTo(value);
                return this;
            },
            isEqual: function (query) {
                if (isPlatformBrowser(PID)) {
                    return db.ref(ref).isEqual(query);
                }
            },
            limitToFirst: function (limit) {
                SQ += "&limitToFirst=" + limit;
                BQ = BQ.limitToFirst(limit);
                return this;
            },
            limitToLast: function (limit) {
                SQ += "&limitToLast=" + limit;
                BQ = BQ.limitToLast(limit);
                return this;
            },
            orderByChild: function (path) {
                SQ += "&orderBy=\"" + path + "\"";
                BQ = BQ.orderByChild(path);
                return this;
            },
            orderByKey: function () {
                SQ += "&orderBy=\"$key\"";
                BQ = BQ.orderByKey();
                return this;
            },
            orderByPriority: function () {
                SQ += "&orderBy=\"$priority\"";
                BQ = BQ.orderByPriority();
                return this;
            },
            orderByValue: function () {
                SQ += "&orderBy=\"$value\"";
                BQ = BQ.orderByValue();
                return this;
            },
            on: function (event) {
                var dataStateKey = makeStateKey(ref + 'rtd');
                if (isPlatformServer(PID)) {
                    return SQH(ref, SQ, dataStateKey);
                }
                if (isPlatformBrowser(PID)) {
                    var SSRedValue = state.get(dataStateKey, []);
                    var VALUE = new BehaviorSubject(SSRedValue);
                    BQ.on(event, BQH(VALUE));
                    return VALUE;
                }
            },
            once: function (event) {
                var dataStateKey = makeStateKey(ref + 'rtd');
                if (isPlatformServer(PID)) {
                    return SQH(ref, SQ, dataStateKey);
                }
                if (isPlatformBrowser(PID)) {
                    var SSRedValue = state.get(dataStateKey, []);
                    var VALUE = new BehaviorSubject(SSRedValue);
                    BQ.once(event, BQH(VALUE));
                    return VALUE;
                }
            }
        };
    };
    AngularFireLiteDatabase.prototype.SRH = function (ref, DSK) {
        var _this = this;
        if (this.server) {
            return this.http.get("https://" + this.config.projectId + ".firebaseio.com/" + ref + ".json")
                .pipe(map(function (payload) {
                if (!!payload && typeof payload === 'object') {
                    var result = Object.keys(payload).map(function (key) {
                        return [payload[key]];
                    });
                    _this.state.set(DSK, result);
                    return result;
                }
                else {
                    _this.state.set(DSK, payload);
                    return payload;
                }
            }));
        }
    };
    AngularFireLiteDatabase.prototype.BRH = function (ref, event, DSK) {
        if (this.browser) {
            var SSRedValue = this.state.get(DSK, []);
            var DATA_1 = new BehaviorSubject(SSRedValue);
            this.ref(ref).on(event, function (snapshot) {
                if (!!snapshot.val() && typeof snapshot.val() === 'object') {
                    var result = Object.keys(snapshot.val()).map(function (key) {
                        return [snapshot.val()[key]];
                    });
                    DATA_1.next(result);
                }
                else {
                    DATA_1.next(snapshot.val());
                }
            });
            return DATA_1;
        }
    };
    AngularFireLiteDatabase = __decorate([
        Injectable(),
        __param(3, Inject(PLATFORM_ID)),
        __metadata("design:paramtypes", [AngularFireLiteApp,
            HttpClient,
            TransferState,
            Object])
    ], AngularFireLiteDatabase);
    return AngularFireLiteDatabase;
}());

var AngularFireLiteFirestore = /** @class */ (function () {
    function AngularFireLiteFirestore(app, http, state, platformId) {
        this.app = app;
        this.http = http;
        this.state = state;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.firestore = app.instance().firestore();
        this.config = app.config();
    }
    // ------------- Read -----------------//
    AngularFireLiteFirestore.prototype.read = function (ref) {
        var _this = this;
        var dataStateKey = makeStateKey(ref + 'firestore:read');
        var refArray = ref.split('/');
        if (refArray[0] === '/') {
            refArray.shift();
        }
        if (refArray[refArray.length - 1] === '/') {
            refArray.pop();
        }
        var slashes = refArray.length - 1;
        if (isPlatformServer(this.platformId)) {
            return this.http
                .get("https://firestore.googleapis.com/v1beta1/projects/" + this.config.projectId + "/databases/(default)/documents/" + ref)
                .pipe(map(function (res) {
                var docData = {};
                if (slashes % 2 !== 0) {
                    Object.keys(res.fields)
                        .forEach(function (key) {
                        for (var keyValue in res.fields[key]) {
                            if (keyValue) {
                                docData[key] = res.fields[key][keyValue];
                            }
                        }
                    });
                    return docData;
                }
                else {
                    var colData_1 = [];
                    res.documents.forEach(function (doc) {
                        var singleDocData = {};
                        Object.keys(doc.fields)
                            .forEach(function (key) {
                            for (var keyValue in doc.fields[key]) {
                                if (keyValue) {
                                    singleDocData[key] = doc.fields[key][keyValue];
                                }
                            }
                        });
                        colData_1.push(singleDocData);
                    });
                    return colData_1;
                }
            }), tap(function (pl) {
                _this.state.set(dataStateKey, pl);
            }));
        }
        if (this.browser) {
            var data_1 = [];
            var SSRedValue = this.state.get(dataStateKey, []);
            var DATA_1 = new BehaviorSubject(SSRedValue);
            if (slashes % 2 === 0) {
                this.firestore.collection(ref).onSnapshot(function (snapshot) {
                    snapshot.docs.forEach(function (doc) {
                        data_1.push(doc.data());
                    });
                    DATA_1.next(data_1);
                });
            }
            else {
                this.firestore.doc(ref).onSnapshot(function (snapshot) {
                    DATA_1.next(snapshot.data());
                });
            }
            return DATA_1;
        }
    };
    // ------------- Write -----------------//
    AngularFireLiteFirestore.prototype.write = function (ref, data, merge) {
        if (this.browser) {
            return from(this.firestore.doc(ref).set(data, { merge: merge }));
        }
    };
    AngularFireLiteFirestore.prototype.push = function (ref, data) {
        if (this.browser) {
            return from(this.firestore.collection(ref).add(data));
        }
    };
    AngularFireLiteFirestore.prototype.update = function (ref, data) {
        if (this.browser) {
            return from(this.firestore.doc(ref).update(data));
        }
    };
    // ------------- Delete -----------------//
    AngularFireLiteFirestore.prototype.remove = function (DocumentRef) {
        if (isPlatformServer(this.platformId)) {
            return this.http
                .delete("https://firestore.googleapis.com/v1beta1/projects/" + this.config.projectId + "/databases/(default)/documents/" + DocumentRef);
        }
        if (this.browser) {
            return from(this.firestore.doc(DocumentRef).delete());
        }
    };
    AngularFireLiteFirestore.prototype.removeField = function (ref) {
        var fields = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fields[_i - 1] = arguments[_i];
        }
        if (this.browser) {
            return from(this.firestore.doc(ref).update(fields));
        }
    };
    AngularFireLiteFirestore.prototype.removeCollection = function (collectionRef) {
        var _this = this;
        if (this.browser) {
            return from(this.firestore.collection(collectionRef).get().then(function (snapshot) {
                snapshot.docs.forEach(function (doc) {
                    _this.firestore.batch().delete(doc.ref);
                });
            }));
        }
    };
    // ------------- Query -----------------//
    AngularFireLiteFirestore.prototype.query = function (ref) {
        var PID = this.platformId;
        var HTTP = this.http;
        var CONFIG = this.config;
        var state = this.state;
        var fs = this.firestore;
        var SSQ = {
            'from': [{ 'collectionId': ref }]
        };
        var SQ = {
            'structuredQuery': SSQ
        };
        function SQHFS(DSK) {
            var data = [];
            return HTTP
                .post("https://firestore.googleapis.com/v1beta1/projects/" + CONFIG.projectId + "/databases/(default)/documents:runQuery", JSON.stringify(SQ))
                .pipe(map(function (res) {
                var e_1, _a;
                var _loop_1 = function (doc) {
                    var documentData = {};
                    Object.keys(doc.document.fields).forEach(function (fieldName) {
                        var fieldType = Object.keys(doc.document.fields[fieldName]);
                        documentData[fieldName] = doc.document.fields[fieldName][fieldType[0]];
                    });
                    data.push(documentData);
                };
                try {
                    for (var res_1 = __values(res), res_1_1 = res_1.next(); !res_1_1.done; res_1_1 = res_1.next()) {
                        var doc = res_1_1.value;
                        _loop_1(doc);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (res_1_1 && !res_1_1.done && (_a = res_1.return)) _a.call(res_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return data;
            }), tap(function (pl) {
                state.set(DSK, pl);
            }));
        }
        function FSD(value) {
            switch (typeof value) {
                case 'boolean':
                    return { 'booleanValue': value };
                case 'string':
                    return { 'stringValue': value };
                case 'number':
                    return { 'doubleValue': value };
                case 'object':
                    return { 'arrayValue': value };
            }
        }
        var SQOB = [];
        var BQ = fs.collection(ref);
        return {
            where: function (document, comparison, value) {
                var SOP = '';
                switch (comparison) {
                    case '<':
                        SOP = 'LESS_THAN';
                        break;
                    case '<=':
                        SOP = 'LESS_THAN_OR_EQUAL';
                        break;
                    case '>':
                        SOP = 'GREATER_THAN';
                        break;
                    case '>=':
                        SOP = 'GREATER_THAN_OR_EQUAL';
                        break;
                    case '==':
                        SOP = 'EQUAL';
                        break;
                }
                SSQ['where'] = { 'fieldFilter': { 'field': { 'fieldPath': document }, 'op': SOP, 'value': FSD(value) } };
                BQ = BQ.where(document, comparison, value);
                return this;
            },
            startAt: function () {
                var startValue = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    startValue[_i] = arguments[_i];
                }
                var SV = [];
                startValue.forEach(function (value) {
                    SV.push(FSD(value));
                });
                SSQ.startAt = {};
                SSQ.startAt.before = true;
                SSQ.startAt.values = SV;
                BQ = BQ.startAt.apply(BQ, __spread(startValue));
                return this;
            },
            startAfter: function () {
                var startValue = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    startValue[_i] = arguments[_i];
                }
                var SV = [];
                startValue.forEach(function (value) {
                    SV.push(FSD(value));
                });
                SSQ.startAt = {};
                SSQ.startAt.before = false;
                SSQ.startAt.values = SV;
                BQ = BQ.startAfter.apply(BQ, __spread(startValue));
                return this;
            },
            endAt: function () {
                var endValue = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    endValue[_i] = arguments[_i];
                }
                var SV = [];
                endValue.forEach(function (value) {
                    SV.push(FSD(value));
                });
                SSQ.endAt = {};
                SSQ.endAt.before = false;
                SSQ.endAt.values = SV;
                BQ = BQ.endAt.apply(BQ, __spread(endValue));
                return this;
            },
            endBefore: function () {
                var endValue = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    endValue[_i] = arguments[_i];
                }
                var SV = [];
                endValue.forEach(function (value) {
                    SV.push(FSD(value));
                });
                SSQ.endAt = {};
                SSQ.endAt.before = true;
                SSQ.endAt.values = SV;
                BQ = BQ.endBefore.apply(BQ, __spread(endValue));
                return this;
            },
            limit: function (limit) {
                SSQ.limit = limit;
                BQ = BQ.limit(limit);
                return this;
            },
            orderBy: function (path, order) {
                var orderBy = {
                    field: {
                        fieldPath: ''
                    }
                };
                orderBy.field.fieldPath = path;
                switch (order) {
                    case 'asc':
                        orderBy['direction'] = 'ASCENDING';
                        break;
                    case 'desc':
                        orderBy['direction'] = 'DESCENDING';
                        break;
                }
                SQOB.push(orderBy);
                BQ = BQ.orderBy(path, order);
                SSQ.orderBy = SQOB;
                return this;
            },
            on: function () {
                var ONDSK = makeStateKey(ref + 'firestore:query');
                if (isPlatformServer(PID)) {
                    return SQHFS(ONDSK);
                }
                if (isPlatformBrowser(PID)) {
                    var SSRedValue = state.get(ONDSK, []);
                    var VALUE_1 = new BehaviorSubject(SSRedValue);
                    BQ.onSnapshot(function (snapshot) {
                        var data = [];
                        snapshot.forEach(function (doc) {
                            data.push(doc.data());
                        });
                        VALUE_1.next(data);
                    });
                    return VALUE_1;
                }
            },
            get: function () {
                var GETDSK = makeStateKey(ref + 'firestore:query');
                if (isPlatformServer(PID)) {
                    return SQHFS(GETDSK);
                }
                if (isPlatformBrowser(PID)) {
                    var data_2 = [];
                    var SSRedValue = state.get(GETDSK, []);
                    var VALUE_2 = new BehaviorSubject(SSRedValue);
                    BQ.get().then(function (snapshot) {
                        snapshot.forEach(function (doc) {
                            data_2.push(doc);
                        });
                        VALUE_2.next(data_2);
                    });
                    return VALUE_2;
                }
            }
        };
    };
    // ------------- Transactions and Batched Writes -----------------//
    // TODO: Add Transactions SSR support
    AngularFireLiteFirestore.prototype.transaction = function () {
        if (this.browser) {
            var fs_1 = this.firestore;
            var transactionToRun_1;
            var readCount_1 = 0;
            var transactions_1 = {
                get: function (ref) {
                    return fs_1.doc(ref).get();
                },
                set: function (ref, data) {
                    return fs_1.doc(ref).set(data);
                }
            };
            return {
                set: function (ref, data) {
                    transactionToRun_1 = transactionToRun_1.then(function () {
                        transactions_1.set(ref, data);
                    });
                    return this;
                },
                get: function (ref) {
                    var _this = this;
                    var getSubject = new Subject();
                    if (readCount_1 > 0) {
                        transactionToRun_1 = transactionToRun_1.then(function () {
                            transactions_1.get(ref).then(function (value) {
                                getSubject.next({ data: value.data(), next: _this });
                            });
                        });
                    }
                    else if (readCount_1 === 0) {
                        transactionToRun_1 = transactions_1.get(ref).then(function (value) {
                            getSubject.next({ data: value.data(), next: _this });
                        });
                    }
                    readCount_1++;
                    return getSubject;
                },
                run: function () {
                    return from(fs_1.runTransaction(function () {
                        return transactionToRun_1;
                    }));
                }
            };
        }
        else {
            return {
                set: function (ref, data) {
                    return this;
                },
                get: function (ref) {
                    return new Subject();
                },
                run: function () {
                    return of();
                }
            };
        }
    };
    // TODO: Add batched writes SSR support
    AngularFireLiteFirestore.prototype.batch = function () {
        if (this.browser) {
            var fs_2 = this.firestore;
            var b_1 = this.firestore.batch();
            return {
                set: function (ref, data) {
                    b_1.set(fs_2.doc(ref), data);
                    return this;
                },
                update: function (ref, data) {
                    b_1.update(fs_2.doc(ref), data);
                    return this;
                },
                delete: function (ref) {
                    b_1.delete(fs_2.doc(ref));
                    return this;
                },
                commit: function () {
                    return from(b_1.commit());
                }
            };
        }
        else {
            return {
                set: function (ref, data) {
                    return this;
                },
                update: function (ref, data) {
                    return this;
                },
                delete: function (ref) {
                    return this;
                },
                commit: function () {
                    return of();
                }
            };
        }
    };
    AngularFireLiteFirestore = __decorate([
        Injectable(),
        __param(3, Inject(PLATFORM_ID)),
        __metadata("design:paramtypes", [AngularFireLiteApp,
            HttpClient,
            TransferState,
            Object])
    ], AngularFireLiteFirestore);
    return AngularFireLiteFirestore;
}());

var AngularFireLiteStorage = /** @class */ (function () {
    function AngularFireLiteStorage(app, platformId) {
        this.app = app;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.storage = app.instance().storage();
    }
    // TODO: Add Server Side Rendering Support Using The GCS REST API
    // ------------- Upload -----------------//
    AngularFireLiteStorage.prototype.upload = function (ref, file, metadata) {
        if (this.browser) {
            return from(this.child(ref).put(file, metadata));
        }
    };
    AngularFireLiteStorage.prototype.uploadString = function (ref, string) {
        if (this.browser) {
            return from(this.child(ref).putString(string));
        }
    };
    // ------------- Download -----------------//
    AngularFireLiteStorage.prototype.getDownloadURL = function (ref) {
        if (this.browser) {
            return from(this.child(ref).getDownloadURL());
        }
    };
    // ------------- Delete -----------------//
    AngularFireLiteStorage.prototype.remove = function (ref) {
        if (this.browser) {
            return from(this.child(ref).delete());
        }
    };
    // ------------- Metadata -----------------//
    AngularFireLiteStorage.prototype.getMetadata = function (ref) {
        if (this.browser) {
            var META_1 = new Subject();
            this.child(ref).getMetadata().then(function (meta) {
                META_1.next(meta);
            });
            return META_1;
        }
    };
    AngularFireLiteStorage.prototype.updateMetadata = function (ref, metadata) {
        if (this.browser) {
            return from(this.child(ref).updateMetadata(metadata));
        }
    };
    AngularFireLiteStorage.prototype.deleteMetadata = function (ref) {
        if (this.browser) {
            return from(this.child(ref).updateMetadata({
                customMetadata: null,
                cacheControl: null,
                contentEncoding: null,
                contentLanguage: null,
                contentType: null
            }));
        }
    };
    AngularFireLiteStorage.prototype.child = function (ref) {
        return this.storage.ref().child(ref);
    };
    AngularFireLiteStorage = __decorate([
        Injectable(),
        __param(1, Inject(PLATFORM_ID)),
        __metadata("design:paramtypes", [AngularFireLiteApp,
            Object])
    ], AngularFireLiteStorage);
    return AngularFireLiteStorage;
}());

var AngularFireLiteMessaging = /** @class */ (function () {
    function AngularFireLiteMessaging(app, platformId) {
        this.app = app;
        this.platformId = platformId;
        this.browser = isPlatformBrowser(this.platformId);
        this.messaging = app.instance().messaging();
    }
    // ------------- Getters -----------------//
    AngularFireLiteMessaging.prototype.instance = function () {
        if (this.browser) {
            return this.messaging;
        }
    };
    AngularFireLiteMessaging.prototype.token = function () {
        if (this.browser) {
            return from(this.messaging.getToken());
        }
    };
    AngularFireLiteMessaging.prototype.tokenRefresh = function () {
        var _this = this;
        if (this.browser) {
            var REFRESH_TOKEN_1 = new Subject();
            this.messaging.onMessage(function () {
                _this.messaging.getToken().then(function (token) {
                    REFRESH_TOKEN_1.next(token);
                });
            });
            return REFRESH_TOKEN_1;
        }
    };
    // ------------- Actions -----------------//
    AngularFireLiteMessaging.prototype.requestPermission = function () {
        if (this.browser) {
            return from(this.messaging.requestPermission());
        }
    };
    AngularFireLiteMessaging.prototype.deleteToken = function (token) {
        if (this.browser) {
            return from((this.messaging.deleteToken(token)));
        }
    };
    AngularFireLiteMessaging = __decorate([
        Injectable(),
        __param(1, Inject(PLATFORM_ID)),
        __metadata("design:paramtypes", [AngularFireLiteApp,
            Object])
    ], AngularFireLiteMessaging);
    return AngularFireLiteMessaging;
}());

function AngularFireLiteAppFactory(config) {
    return new AngularFireLiteApp(config);
}
// @dynamic
var AngularFireLite = /** @class */ (function () {
    function AngularFireLite() {
    }
    AngularFireLite_1 = AngularFireLite;
    AngularFireLite.forRoot = function (fireConfig) {
        return {
            ngModule: AngularFireLite_1,
            providers: [
                { provide: FirebaseAppConfig, useValue: fireConfig },
                {
                    provide: AngularFireLiteApp,
                    useFactory: AngularFireLiteAppFactory,
                    deps: [FirebaseAppConfig]
                },
                AngularFireLiteFirestore,
                AngularFireLiteDatabase,
                AngularFireLiteAuth,
                AngularFireLiteStorage,
                AngularFireLiteMessaging
            ]
        };
    };
    var AngularFireLite_1;
    AngularFireLite = AngularFireLite_1 = __decorate([
        NgModule({
            imports: [
                HttpClientModule,
                CommonModule,
                BrowserTransferStateModule
            ]
        })
    ], AngularFireLite);
    return AngularFireLite;
}());

export { AngularFireLite, AngularFireLiteApp, AngularFireLiteAppFactory, AngularFireLiteAuth, AngularFireLiteDatabase, AngularFireLiteFirestore, AngularFireLiteMessaging, AngularFireLiteStorage, FirebaseAppConfig as ɵa };
//# sourceMappingURL=angularfire-lite.js.map
